from django.db import models
from django.conf import settings
from django.contrib.auth.models import User
from django.template.defaultfilters import slugify
import os
from stager.widgets import ColorPickerWidget

class ColorField(models.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 10
        super(ColorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['widget'] = ColorPickerWidget
        return super(ColorField, self).formfield(**kwargs)

try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^stager\.staging\.models\.ColorField"])
except ImportError:
    pass
    
def get_slide_path(instance, filename):
    return settings.FILE_UPLOAD + "/" + \
        instance.linked.project.client.path + "/" + \
        instance.linked.project.path + "/comps/" + filename

def get_logo_path(instance, filename):
    return settings.FILE_UPLOAD + "/" + instance.path + "/" + filename

def get_link_path(instance, filename):
    return settings.FILE_UPLOAD + "/" + \
        instance.project.project.client.path + "/" + \
        instance.project.project.path + filename

class Client(models.Model):
    """
    A client.
    """
    name = models.CharField(max_length=100)
    active = models.BooleanField(default=True)
    path = models.CharField('URL fragment', 
        help_text='<a href="' + settings.BASE_URL + 'url_fragment/" \
                   target="_new" id="url_fragment">' + settings.BASE_URL + 'url_fragment/</a>', max_length=100, unique=True)
    logo = models.ImageField(upload_to=get_logo_path, help_text="( Width = 200, Height = 55 )", blank=True, null=True)
    user = models.ForeignKey(User,related_name='users')
    contact = models.ForeignKey('Contact', related_name='clients',  blank=True, null=True, 
        help_text='The PM. (shows up on the internal homepage)')

    def __unicode__(self):
        return self.name
    
    @property
    def has_multiple_projects(self):
        """
        Returns true if there is more than one active project associated with this client.
        """
        return self.projects.filter(active=True).count() > 1
    
    @property
    def active_projects(self):
        """
        Returns the active projects for the client.
        """
        return self.projects.filter(active=True)
        
    def link(self):
        return "/" + self.path + "/"

class Container(models.Model):
    name = models.CharField(blank=False, max_length=100)
    description = models.TextField(blank=True)
    active = models.BooleanField(default=True)
    
    def __unicode__(self):
        return self.name
    
class Project(Container):
    """
    A project.
    """
    client = models.ForeignKey(Client, related_name='projects')
    path = models.CharField('URL fragment',
        help_text=settings.BASE_URL + 'client/url_fragment/', max_length=100)
    
    class Meta:
        ordering = ['name',]

    def __unicode__(self):
        return '%s - %s' % (self.client.name,self.name)
    
    @property
    def ai_contacts(self):
        return self.contacts.filter(ai_staff=True).filter(active=True)
    
    @property
    def client_contacts(self):
        return self.contacts.filter(ai_staff=False).filter(active=True)
    
    @property
    def active_sections(self):
        return self.sections.filter(active=True).order_by('-date')
    
    #@property
    ##def category_rollups(self):
      #  return [CategoryRollup(category,self.active_sections.filter(category=category)) for category in self.categories.all()]

    @property
    def link(self):
        return "/clients/" # + self.client.path + "/" + self.path + "/"

class Contact(models.Model):
    """
    A contact on the project.
    """
    first_name = models.CharField(blank=True, max_length=100)
    last_name = models.CharField(blank=True, max_length=100)
    telephone = models.CharField(blank=True, max_length=100)
    title = models.CharField(blank=True, max_length=100)
    email = models.EmailField()
    projects = models.ManyToManyField(Project,related_name='contacts')
    active = models.BooleanField(default=True)
    ai_staff = models.BooleanField()
    ordering = models.IntegerField(default=0)

    def __unicode__(self):
        return '%s %s' % (self.first_name,self.last_name)

    class Meta:
        ordering = ['ordering']

class Section(Container):
    """
    A section of a project.
    """
    project = models.ForeignKey(Project,related_name='sections')
    date = models.DateField(verbose_name='Date Modified', auto_now_add=True, auto_now=True)
    #category = models.ForeignKey(Category)
    ordering = models.IntegerField(default=0)

    def __unicode__(self):
        return self.name
    
    @property
    def active_links(self):
        return self.links.filter(active=True)
    
    class Meta:
        ordering = ['ordering']

class Link(models.Model):
    """
    A link to a file.
    """
    remote_link = models.URLField(blank=True, null=True, verify_exists=False)
    file_link = models.FileField(upload_to=get_link_path,blank=True,null=True)
    linked = models.ForeignKey(Container, related_name="links", blank=False, null=False)
    name = models.CharField(blank=True, max_length=100)
    active = models.BooleanField(default=True)
    ordering = models.IntegerField(default=0)
    
    def __unicode__(self):
        return self.name
    
    @property
    def link(self):
        if self.file_link:
            return '%s%s' % (settings.MEDIA_URL, self.file_link)
        else:
            return self.remote_link

    @property
    def project(self):
        try:
            return self.linked.section.section.project
        except:
            try:
                return self.linked.subsection.section.section.project
            except:
                return self.linked.project

    class Meta:
        ordering = ['ordering', ]

class CompSlide(models.Model):
    ALIGNMENT_CHOICES = (
        ('left','left'),
        ('right','right'),
    )
    name = models.CharField(max_length=128)
    linked = models.ForeignKey(Container, related_name="slides", blank=False, null=False)
    image = models.ImageField(upload_to=get_slide_path, max_length=256, blank=True, null=True)
    active = models.BooleanField(default=True)
    ordering = models.IntegerField(default=0)
    date = models.DateField(verbose_name='Date Modified', auto_now_add=True, auto_now=True)
    
    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ['ordering', ]

    @property
    def link(self):
        section = self.linked.section
        project = section.project
        return '/client/%s/%s/comps/%s/' % (project.client.path, project.path, self.id )

    @property
    def returnlink(self):
        section = self.linked.section
        project = section.project
        return '/client/%s/%s/' % (project.client.path,  project.path)

class ViewChoice(models.Model):
    """Stores a user's display choice - grid or list"""
    default_d = models.TextField(choices = (('grid', 'grid'), ('list','list'))) 
     
    def __unicode__(self):
        return self.default_d

class UserPreference(models.Model):
    """This class will add a feature to Django's User class using a connecting one-to-one field on the User Model"""
    default_display = models.ForeignKey(ViewChoice, default = ViewChoice.objects.get(id=1).id) # keep grid the initial view
    user = models.OneToOneField(User, unique=True, related_name='userschoices')

    def __unicode__(self):
	return self.user.username + "'s viewing choice"

from django.db.models.signals import post_save
def create_userpreference(sender, instance, created, **kwargs):
    if created:
	UserPreference.objects.get_or_create(user=instance)

post_save.connect(create_userpreference, sender=User)

